package adminkindboxvalidator

import (
	"errors"

	param "git.gocasts.ir/ebhomengo/niki/param/admin/kind_box"
	errmsg "git.gocasts.ir/ebhomengo/niki/pkg/err_msg"
	richerror "git.gocasts.ir/ebhomengo/niki/pkg/rich_error"
	"git.gocasts.ir/ebhomengo/niki/validator"
	validation "github.com/go-ozzo/ozzo-validation/v4"
)

func (v Validator) ValidateGetAll(req param.KindBoxGetAllRequest) (map[string]string, error) {
	const op = "adminkindboxvalidator.ValidateGetAll"

	validFields := []string{
		"id", "kind_box_req_id", "benefactor_id", "kind_box_type", "amount", "serial_number",
		"status", "deliver_refer_time_id", "deliver_refer_date", "deliver_address_id", "sender_agent_id",
		"delivered_at", "return_refer_time_id", "return_refer_date",
		"return_address_id", "receiver_agent_id", "returned_at",
	}
	if err := validation.ValidateStruct(&req,
		validation.Field(&req.Filter, validation.By(validator.AreFilterFieldsValid(validFields))),
		validation.Field(&req.Sort, validation.By(validator.AreSortFieldsValid(validFields))),
	); err != nil {

		fieldErrors := make(map[string]string)

		var errV validation.Errors
		if errors.As(err, &errV) {
			for key, value := range errV {
				if value != nil {
					fieldErrors[key] = value.Error()
				}
			}
		}

		return fieldErrors, richerror.New(op).
			WithMessage(errmsg.ErrorMsgInvalidInput).
			WithKind(richerror.KindInvalid).
			WithMeta(map[string]interface{}{"req": req}).
			WithErr(err)
	}

	return map[string]string{}, nil
}
